# -*- coding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType

from django.db import models
from django.contrib.auth.models import User


class Application(models.Model):
    name = models.CharField(max_length=60, unique=True, verbose_name=u"nome")
    python_name = models.CharField(max_length=255, unique=True, verbose_name=u"Nome no Python")
    managers = models.ManyToManyField(User, verbose_name=u'gestores', related_name='managed_applications')
    enabled = models.BooleanField(default=True, verbose_name=u"ativa")

    def __unicode__(self):
        return self.name

    url_name = 'aplicacao'

    class Meta(object):
        verbose_name = u'aplicação'
        verbose_name_plural = u'aplicações'


class Profile(models.Model):
    id = models.AutoField(primary_key=True)
    application = models.ForeignKey(Application, verbose_name=u'aplicação')
    name = models.CharField(max_length=60, verbose_name=u'nome')
    description = models.TextField(verbose_name=u'descrição')
    users = models.ManyToManyField(User, verbose_name=u'usuários', blank=True)

    def __unicode__(self):
        return self.name

    class Meta(object):
        verbose_name = u'perfil'
        verbose_name_plural = u'perfis'


class DocumentTemplate(models.Model):
    name = models.CharField(max_length=60, verbose_name=u'nome')
    model = models.ForeignKey(ContentType)
    template = models.FileField(upload_to='rapid/document_templates')

    def __unicode__(self):
        return self.name

    class Meta(object):
        verbose_name = u'template de documento'
        verbose_name_plural = u'templates de documento'
